#!/usr/bin/env python3
#-------------------------------------------------------------------------------
#   Author:     Magnus Overbo
#   Date:       20190324
#   Version:    1
#   Description:Opens RS232 communication to read 32b values sent from an FPGA.
#               It parses the data and performs synchronisation.  When the data
#               repeats itself, it will automatically stop receiving data and 
#               print its data to file and screen.
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
#   Library import
#-------------------------------------------------------------------------------
import serial           #UART / RS232 library
import sys              #System
import datetime
from Crypto.Util.number import bytes_to_long   #Convert bytestring to long


#-------------------------------------------------------------------------------
#   Global variables
#-------------------------------------------------------------------------------
JSONDATA    = {}
i           = 0
POS         = None
STT         = None
ERR         = None
RUN         = True
DELAY       = False
LFSRSIZE    = int(sys.argv[1])
M           = int(sys.argv[2])
K           = int(sys.argv[3])
C           = int(sys.argv[4])
DSTART      = None
DEND        = None
LINIT       = -1
                    #Open COM port for UART communication at 115200 baud rate
serialConnection = serial.Serial("/dev/ttyUSB1", 115200, timeout=1/1000)


#-------------------------------------------------------------------------------
#   Helper function for printing binary representation of data
#-------------------------------------------------------------------------------
def binp( a, l ):
  A  = "{:b}".format( a )
  L  = l - len(A)
  return "{}{}".format("0"*L, A)


#-------------------------------------------------------------------------------
#   Main loop 
#-------------------------------------------------------------------------------
while RUN:
  DATA = serialConnection.read(1)              #Read 1 byte
  if len(DATA) == 0 or len(DATA) > 2 or DELAY: #Data synch (Skip one byte)
    DELAY   = False
    continue

  #Load 32b data to generate long data
  if i < 4:
    POS = DATA if i == 0 else DATA + POS
  elif i < 8:
    STT = DATA if i == 4 else DATA + STT
  elif i < 12:
    ERR = DATA if i == 8 else DATA + ERR

  if i == 11:                                  #Convert and store data
    STT = bytes_to_long(STT)
    POS = bytes_to_long(POS)
    ERR = bytes_to_long(ERR)

    if ERR > K:                                #if ERR > K, means byte mismatch
      JSONDATA  = {}                           #  easiest to remove and redo
      DELAY     = True                         #  Force delay to resynch
      i         = 0
      DSTART    = None
      DEND      = None
      sys.stdout.write("Error: {}\t{}\t{}\n".format( binp(STT,10), POS, ERR) )
      sys.stdout.write("Resetting collected data, and skipping: 1B\n\n")
      continue

    if LINIT != STT:                           #New LFSR state logged
      sys.stdout.write("STATE: {:>10}\t{}\n".format(STT, binp(STT,LFSRSIZE)))
      LINIT     = STT

    if DSTART == None:
      DSTART = datetime.datetime.now()
    
    if STT in JSONDATA.keys():                 #Add data to array
      if POS in JSONDATA[STT].keys():
        DEND    = datetime.datetime.now()
        RUN     = False                        #End
      else:
        JSONDATA[STT].update( { POS:ERR } )    #new position
    else:
      JSONDATA.update( { STT : {POS:ERR} } )   #Entire new
  i = (i + 1) % 12                             #Mod counter
serialConnection.close()                       #Close connection to UART RS232


#-------------------------------------------------------------------------------
#   Output data data to file and console
#-------------------------------------------------------------------------------
TDELTA = DEND-DSTART
PSTR="fpga_L{}M{}K{}C{}_{}.log"
fh = open(PSTR.format(LFSRSIZE, M, K, C, DSTART.strftime('%Y%m%d%H%M%S')), "w")
for key in sorted(JSONDATA.keys()):
  sys.stdout.write( "{}\t{}\t".format( key, binp(key,LFSRSIZE) ) )

  sys.stdout.write("\nMATCH_COUNT: {}\n".format( len(JSONDATA[key].keys()) ) )
  fh.write("\nMATCH_COUNT: {}\n".format( len(JSONDATA[key].keys()) ) )
  
  fh.write("INITSTATE {}\t{}\nMATCH\t".format( key, binp(key,LFSRSIZE) ) )
  for skey in sorted(JSONDATA[key].keys()):
    sys.stdout.write("{} ".format(skey))
    fh.write( "{}:{} ".format(skey, JSONDATA[key][skey]))
  sys.stdout.write( "\n\n" )
  fh.write( "\n\n" )

fh.write("START:\t{}\nEND:\t{}\nTOT:\t{}\n".format(
        DSTART.strftime('%Y %m %d %H:%M:%S.%f'),
        DEND.strftime('%Y %m %d %H:%M:%S.%f'),
        TDELTA)
    )
sys.stdout.write("START:\t{}\nEND:\t{}\nTOT:\t{}\n".format(
        DSTART.strftime('%Y %m %d %H:%M:%S.%f'),
        DEND.strftime('%Y %m %d %H:%M:%S.%f'),
        TDELTA)
    )
    

fh.close()
